;
; 12/24V Fan regulator with alarm
; Piotr Wjtowicz
; 2016-2017
;
;#define TEMP_AUTOINCR				; Auto increment and decr temp_real register, only for debugging
;#define MEASURE				; Measurement of rotation
;#define PRINT_UART				; Sending registers over Uart only for debuging/diagnostics
    
    radix		DEC	
    processor		10f202
    
 __CONFIG _MCLRE_OFF & _CP_OFF & _WDT_ON

; The pinology ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
DS18B20_PIN		equ 0
CAP_PIN			equ 1
ALARM_PIN		equ 2
FANMEASURE_PIN		equ 3
COM_TX			equ ALARM_PIN
	    
; General purpose regs: 08h..1Fh ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
tris_reg		equ	    H'08'
delay1_cnt		equ	    H'09'	; register used in delay function			
delay2_cnt		equ	    H'0A'	; register used in delay function
fsreg			equ	    H'0B'	; fan speed pulse
fsregt			equ	    H'0C'	; fan speed rotation register
tmpcnt			equ	    H'0D'	; debug counter to simulation changing of temperature
state_reg		equ	    H'0E'	; state machine board
    errorlevel -207		
    PRESENCE_FLAG	equ 0	;bit 0 - PRESENCE_FLAG ostatni device presence 0:brak 1:jest
    MATCH_DSID_FLAG	equ 1	;bit 1 - MATCH_DSID_FLAG ostatni device ID 0:nok 1:ok
    MATCH_CRC_FLAG	equ 2	;bit 2 - MATCH_CRC_FLAG ostatnie crc 0:nok 1:ok
    SERIAL_ENABLED	equ 3	;bit 3 - SERIAL_ENABLED serial data enabled
    FAN_ROT		equ 4	;bit 4 - FAN_ROT wentylator kreci sie 0:nie 1:tak
    ALARM_ON		equ 5	;bit 5 - ALARM_ON alarm na wyjsciu ukladu 0:off 1:on
    TMPCNT		equ 6	;bit 6 - kierunek zliczania tmpcnt; 0-gora, 1-dol
    TEMP_OVER		equ 7  ;bit 7 - alarm przekroczenia temperatury
    errorlevel +207
tmp1			equ	    H'0F'	; id odczytane z ROM DSa
IOBYTE			equ	    H'10'
tmp0			equ	    H'11'	; Address 0x23
ONE			equ	    H'12'	; Keep track of bits
#define COUNT tmp0
TMSB			equ	    H'13'	; Store the MSB
TLSB			equ	    H'14'	; Store the LSB
temp			equ	    H'15'	; temperature
COM_BUF			equ	    H'16'	; Bufor danych wysy?anych przez COM
COM_PTR			equ	    H'17'	; Pointer na bit wysy?anego bajta
cnt1			equ	    H'18'
cnt2			equ	    H'19'
tcharge			equ	    H'1A'
temp_real		equ	    H'1B'
crc_reg			equ	    H'1C'
crc_input		equ	    H'1D'
crc_code		equ	    H'1E'
delay3_cnt		equ	    H'1F'	; tris register state

; Constans ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
optreg			equ	    b'11001111'	    ; wdt prescaler 128   
gpio_masq		equ	    (1 << DS18B20_PIN) | (1 << CAP_PIN) | (1 << FANMEASURE_PIN) ; input pins masq 1 = in; 0 = out
tfmin			equ	    35		    ; min temperature of f(t)charge function
tfmax			equ	    (tfmin + 30)    ; max temperature of f(t)charge function
talh			equ	    75		    ; temperature of trigger alarm
tall			equ	    50		    ; temperature off of alarm
tdef			equ	    25		    ; default temperature
STX			equ	    2
ETX			equ	    3

; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
RES_VECT	code    0x0000
    GOTO		init
MAIN_PROG	code				    ; let linker place main program      

; Includes ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;	
#include <p10f202.inc>				    ; processor specific variable definitions
#include macros.inc
#include ow.inc

#ifdef	TEMP_AUTOINCR
#include debug.inc
#endif	

#include fan.inc
#ifdef	PRINT_UART
#include com.inc
#endif	
#include alarm.inc

init: ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    CLRWDT
    MOVLW	optreg			    ; temporary assign prescaler to T0
    OPTION    
    MOVLW	gpio_masq
    TRIS 	GPIO			    ; set default pins configuration
    
    CLRF	ONE
    INCF	ONE,F
    
    SetAlarmPin				    ; alarm output active at initial
    SetPinH	CAP_PIN			    ; full cap charging
    DelayUs	2000000		    ; delay for hw test/alarm
    CLRWDT
    DelayUs	2000000		    ; delay for hw test/alarm
    ClearAlarmPin			    ; clear alarm output
    CLRF	state_reg
    
        
MainLoop: ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    CLRWDT
    MOVLW	(optreg & b'11111101')	    ; set wdt prescaler 32 (~0.576s)
    OPTION
    
    GOTO	READ_TYPE_DS_LAB
READ_TYPE_DS_LAB_ret:
    BCF		state_reg, MATCH_DSID_FLAG 
    MOVLW	DS18B20_ID
    SUBWF	tmp1,W
    BTFSC	STATUS,Z		    ; check if id code valid
    BSF		state_reg, MATCH_DSID_FLAG 
    GOTO	CONFIG_DS_LAB
CONFIG_DS_LAB_END:
    GOTO	CONVERT_TEMP_LAB
CONVERT_TEMP_LAB_END:
    GOTO	GET_TEMP_LAB
GET_TEMP_LAB_END: 
    
    GOTO	DS18B20_TEMP_CONV_LAB
DS18B20_TEMP_CONV_LAB_ret:    

#ifdef TEMP_AUTOINCR    
    GOTO	debug_procedure		    ; debug procedure - increment/decrement temperature
#endif    
debug_procedure_ret:    
    
    GOTO	NORMALIZE_TEMP_LAB
NORMALIZE_TEMP_LAB_ret:

    GOTO	alarm_handle
alarm_handle_ret:    
    
    GOTO	FAN_REG_LAB
FAN_REG_LAB_end:

#ifdef PRINT_UART    
    GOTO	print_data
#else    
    GOTO	output_procedure
#endif 
    
print_data_end:            
output_procedure_ret:    
    
fan_measure_end:    
    GOTO	MainLoop
    

    END