SRCHROM		    equ 0xF0
RDROM		    equ 0x33
MTCHROM		    equ 0x55
SKPROM		    equ 0xCC
DS18B20_ID	    equ 0x28

OWHZ_FUN:
    BSF		tris_reg, DS18B20_PIN
    MOVF	tris_reg,W
    TRIS	GPIO
    RETLW	0
OWL_FUN:
    BCF		GPIO, DS18B20_PIN
    BCF		tris_reg, DS18B20_PIN
    MOVF	tris_reg,W
    TRIS	GPIO
    RETLW	0
    
READ_TYPE_DS_LAB ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    CLRF	crc_reg
    CALL        OW_RESET_FUN                ; Send Reset Pulse and read for Presence Detect Pulse
    MOVLW	0x33			    ; Read On-Board ROM
    CALL	DSTXBYTE_FUN		    ; send command to DS
    CALL        DSRXBYTE_FUN		    ; Byte 0 from On-Board ROM - device ID
    MOVF        IOBYTE,W		    ; Save device_id
    MOVWF       tmp1
    MOVLW	7			    ; Reading remain seven bytes from On-Board ROM
    MOVWF	cnt2
READ_TYPE_DS_LAB_loop1:
    CALL        DSRXBYTE_FUN
    DECFSZ	cnt2,F
    GOTO	READ_TYPE_DS_LAB_loop1
    BCF		state_reg, MATCH_CRC_FLAG
    MOVF	crc_reg,f		    ;
    BTFSC	STATUS,Z		    ; If crc_reg is zero then MATCH_CRC_FLAG is true
    BSF		state_reg, MATCH_CRC_FLAG    
    GOTO	READ_TYPE_DS_LAB_ret

CONFIG_DS_LAB: ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    CALL	OW_RESET_FUN		    ; Send Reset Pulse and read for Presence Detect Pulse
    MOVLW	0xCC			    ; Skip ROM
    CALL	DSTXBYTE_FUN
    MOVLW	0x4E			    ; Write scratchpad
    CALL	DSTXBYTE_FUN		    ; Send function command
    CALL	DSTXBYTE_FUN		    ; Send TL
    CALL	DSTXBYTE_FUN		    ; Send TH
    MOVLW	0x1F			    ; 0b00011111 - 9bit resolution
    CALL	DSTXBYTE_FUN		    ; Send 3rd scratch byte (conf)
    GOTO	CONFIG_DS_LAB_END

CONVERT_TEMP_LAB: ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    CALL	OW_RESET_FUN		    ; Send Reset Pulse and read for Presence Detect Pulse
    MOVLW	0xCC			    ; Skip ROM
    CALL	DSTXBYTE_FUN
    MOVLW	0x44
    CALL	DSTXBYTE_FUN		    ; Send T conv
    OwOutH				    ; Strong pull-up on - not required while non-parasite supply
    DelayUs	141205			    ; wait 141205us
    CALL	OWHZ_FUN		    ; Strong pull-up off (if parasite supply)
    GOTO	CONVERT_TEMP_LAB_END

GET_TEMP_LAB: ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    CLRF	crc_reg
    CALL	OW_RESET_FUN		    ; Send Reset Pulse and read for Presence Detect Pulse
    MOVLW	0xCC
    CALL	DSTXBYTE_FUN
    MOVLW	0xBE			    ; Read scratchpad
    CALL	DSTXBYTE_FUN		    ; Send command to DS
    CALL	DSRXBYTE_FUN		    ; Byte 0 - Temp LSB
    MOVF	IOBYTE,W
    MOVWF	TLSB			    ; LSB temperature
    CALL	DSRXBYTE_FUN		    ; Byte 1 - Temp MSB
    MOVF	IOBYTE,W
    MOVWF	TMSB			    ; MSB temperature
    MOVLW	.7
    MOVWF	cnt2
gt_loop:
    CALL        DSRXBYTE_FUN
    DECFSZ	cnt2,F
    GOTO	gt_loop
    BCF		state_reg, MATCH_CRC_FLAG
    MOVF	crc_reg,f		    
    BTFSC	STATUS,Z		    ;if crc_reg is zero then MATCH_CRC_FLAG is true
    BSF		state_reg, MATCH_CRC_FLAG 
    GOTO	GET_TEMP_LAB_END

OW_RESET_FUN: ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    CALL	OWHZ_FUN
    DelayUs	1403			    ; wait 1403us
    BCF		state_reg, PRESENCE_FLAG    ; Clear DS presence flag
    CALL	OWL_FUN
    DelayUs	353			    ; wait 353us
    CALL	OWHZ_FUN
    DelayUs	52			    ; wait 52us
    BTFSS       GPIO,DS18B20_PIN	    ; Read for a PD Pulse
    BSF		state_reg,PRESENCE_FLAG	    ; Set DS presence flag
    DelayUs	283			    ; wait 283us
    RETLW 0

DSRXBYTE_FUN: ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    MOVLW       8			    ; Byte read is stored in IOBYTE
    MOVWF       tmp0                       ; Set tmp0 equal to 8 to count the bits
dsrx_loop:
    CALL	OWL_FUN
    DelayUs	6
    CALL	OWHZ_FUN
    DelayUs	8
    MOVF        GPIO,W			    ; Read DQ
    ANDLW	(1 << DS18B20_PIN)          ; Mask off the DQ bit
    MOVWF	cnt1
    MOVLW	.255
    ADDWF	cnt1,F
    RRF         IOBYTE,F                    ; Shift C into IOBYTE
    DelayUs	38
    DECFSZ      tmp0,F                     ; Decrement the bit counter
    GOTO        dsrx_loop
    MOVF	IOBYTE,W		    ; copy IOBYTE...
    GOTO	CRC_CALC_FUN		    ; and append it to crc calculation 
CRC_CALC_FUN_ret:
    RETLW	0

DSTXBYTE_FUN: ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    MOVWF	IOBYTE                      ; Byte to send starts in W and copy to IOBYTE
    MOVLW	8
    MOVWF	tmp0                       ; Set tmp0 equal to 8 to count the bits
dstx_loop:
    CALL	OWL_FUN
    CLRW    
    MOVLW	gpio_masq & ~(1 << DS18B20_PIN)
    RRF		IOBYTE,F
    BTFSC	STATUS,C                    ; Check the LSB of IOBYTE for 1 or 0
    MOVLW	gpio_masq | (1 << DS18B20_PIN)
    TRIS	GPIO
    DelayUs	45			    ; wait 45us
    CALL	OWHZ_FUN		    ; Release the line for pullup
    DelayUs	2
    DECFSZ	tmp0,F                     ; Decrement the bit counter
    GOTO	dstx_loop
    RETLW	0

DS18B20_TEMP_CONV_LAB: ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    MOVF	TMSB,W
    MOVWF	cnt1
    MOVF	TLSB,W
    MOVWF	cnt2
    RLF		cnt1,F			    ; Temperature 18B20 sensor conversion
    RLF		cnt1,F
    RLF		cnt1,F
    RLF		cnt1,F
    RLF		cnt1,F
    RRF		cnt2,F
    RRF		cnt2,F
    RRF		cnt2,F
    MOVF	cnt1,W			    ; temp = ((th << 5) | (tl >> 3)) - (tmin * 2)
    ANDLW	b'11100000'
    MOVWF	temp_real
    MOVF	cnt2,W
    ANDLW	b'00011111'

    IORWF	temp_real,F
    GOTO	DS18B20_TEMP_CONV_LAB_ret

NORMALIZE_TEMP_LAB: ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;    
    MOVF	temp_real,W
    ;MOVLW	(35 * 2)		    ; for testing
    ;MOVLW	(50 * 2)		    ; for testing
    ;MOVLW	(65 * 2)		    ; for testing
    MOVWF	temp
    MOVLW	(.2 * tfmin)		    ; temp = max(temp, tl)
    SUBWF	temp,F
    BTFSS	STATUS,C
    CLRF	temp
    ADDWF	temp,F
    MOVLW	(.2 * talh)		    ; temp = min(temp, tah)
    SUBWF	temp,F
    BTFSC	STATUS,C
    CLRF	temp
    ADDWF	temp,F
    GOTO	NORMALIZE_TEMP_LAB_ret    
    
CRC_CALC_FUN: ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    MOVF	IOBYTE,W
    MOVWF	crc_input
    MOVLW	.8
    MOVWF	cnt1
ccf_loop:
    MOVF	crc_reg,W
    BCF		STATUS,C
    RRF		crc_reg,F
    XORWF	crc_input,W
    ANDLW	0x01
    BTFSC	STATUS,Z
    GOTO	ccf_next_byte
    MOVLW	0x8C
    XORWF	crc_reg,F
ccf_next_byte:        
    RRF		crc_input,F
    DECF	cnt1,F
    BTFSS	STATUS,Z
    GOTO	ccf_loop
ccf_end:
    GOTO	CRC_CALC_FUN_ret    